freeze;


function MakeSolutionPrimitive3(v)

  //  transforms solution of the quadratic form such that 
  //  its components are coprime (no integer divides all of them) 

  if (v eq Vector([0,0,0])) then
    return v;
  end if;

  gcd := GCD(v[1], GCD(v[2], v[3]));
  if (gcd ne 1) then
    v[1] := v[1] div gcd;
    v[2] := v[2] div gcd;
    v[3] := v[3] div gcd;
  end if;

  return v;
end function;


intrinsic IsIsotropic3(
  coef1::RngIntElt, coef2::RngIntElt, coef3::RngIntElt :
  Al := "CremonaRusin") -> BoolElt, ModTupRngElt
{ Returns whether the ternary quadratic form is isotropic over rationals 
 and isotropic vector with integral coordinates if so }

  // solve trivial case: zero occures on diagonal
  if (coef1 eq 0) then
    return true, Vector([1,0,0]);
  elif (coef2 eq 0) then
    return true, Vector([0,1,0]);
  elif (coef3 eq 0) then
    return true, Vector([0,0,1]);
  end if;

  //  simple test of solvability
  if ((coef1 gt 0) and (coef2 gt 0) and (coef3 gt 0)) or 
     ((coef1 lt 0) and (coef2 lt 0) and (coef3 lt 0)) then 
    return false, _; 
  end if;

  P2<x,y,z> := ProjectivePlane(Rationals());
  C := Conic(P2, coef1 * x^2 + coef2 * y^2 + coef3 * z^2);
  b, pt := HasRationalPoint(C);
  if (not b) then
    return false, _;
  end if;

  d := Lcm(Lcm(Denominator(pt[1]), Denominator(pt[2])), Denominator(pt[3]));
  return true, Vector([Numerator(pt[1])*d div Denominator(pt[1]),
                       Numerator(pt[2])*d div Denominator(pt[2]),
                       Numerator(pt[3])*d div Denominator(pt[3])]);

end intrinsic;



intrinsic IsIsotropic3(QF_M::AlgMatElt : 
  Al := "CremonaRusin") -> BoolElt, ModTupRngElt
{ Returns whether the ternary quadratic form is isotropic over rationals 
 and isotropic vector with integral coordinates if so }

  require IsSymmetric(QF_M): "Symmetric matrix required.";
  require NumberOfRows(QF_M) eq 3: "3 x 3 matrix required.";

  QF_M1, T, rank := OrthogonalizeGram(QF_M);

  b, v := IsIsotropic3(QF_M1[1][1], QF_M1[2][2], QF_M1[3][3] : Al := Al);
  if b then 
    return true, MakeSolutionPrimitive3(v * T);
  end if;

  return false, _;

end intrinsic;



